# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2024 CERN.
#
# Babel-EDTF is free software; you can redistribute it and/or modify it under
# the terms of the MIT License; see LICENSE file for more details.

"""Version information for Babel-EDTF.

This file is imported by ``babel_edtf.__init__``,
and parsed by ``setup.py``.
"""

__version__ = '1.2.0'
